/*
	
 *************************************************************************************
 *                                                                                   *
 *               	MIRA - Multiple Infra Red Application                            *
 *                                                                                   *
 *************************************************************************************
 *                                                                                   *
 *  File:        	MIRA.c                                                           *
 *  Description:    Main C file contains rutines to hardware connection of MIRA unit *                                                                   *
 *                                                                                   *
 *  Author:   	 	Bc. Miroslav Skopal                                              *
 *  Date:     		23.5.2009                                                        *
 *  Platform: 		AVR (Tested on ATMega16 (fcpu = 16MHz)                           *
 *                                                                                   *
 *  License:		Licensed under GPLv2                                             *
 *                                                                                   *
 *  MIRA - Multiple InraRed Application Copyright (C) 2009 Miroslav Skopal           *
 *                                                                                   *
 *  This program is free software; you can redistribute it and/or                    *
 *  modify it under the terms of the GNU General Public License                      *
 *  as published by the Free Software Foundation; either version 2                   *
 *  of the License.                                                                  *
 *                                                                                   *
 *  This program is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 *  GNU General Public License for more details.                                     *
 *                                                                                   *
 *  You should have received a copy of the GNU General Public License                *
 *  along with this program; if not, write to the Free Software                      *
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 *                                                                                   * 
 *************************************************************************************

*/



#include <avr/io.h>
#include <avr/interrupt.h>
#include "MIRA.h"

// preruseni reagujici na sestupnou hranu na pinu ICP1/PD6
ISR(TIMER1_CAPT_vect ){	
	// vynulovani citace 
	TCNT1 = 0;	 // magicke cislo 48 :)
	
	// roznuti indikacni diody prijmu
	PORTD |= _BV(PD4);
	
	// doba mezi prijeti po sobe jdoucich impulsu v mikrosekundach
	unsigned short int us;
	
	// pocet impulzu mezi roznutim IR diaody (plati pro prescaler F/1
	us = ICR1/(F_CPU/1000000); 
	
	// vyvolani callback funkci
	#ifdef MIRACallback
		MIRACallback;
	#else
		# warning : MIRACallback not defined for "MIRA.h"
	#endif


	// zhasnuti indikacni diody prijmu
	PORTD &= ~(_BV(PD4));

}



void MIRA_Init(){
	
	// nastaveni portu
	IR_TX_DDR |= _BV(IR_TX_PIN);
	IR_RX_DDR &= ~(_BV(IR_RX_PIN)); // nastaveni RX pinu jako vstup

	//IR_RX_DDR |= _BV(IR_RX_PIN); // nastaveni RX ponu jako vystup
	
//	IR_RX_PORT |= _BV(IR_RX_PIN);

	TCCR1B &= ~(_BV(ICNC1)); // filtr
	TCCR1B &= ~(_BV(ICES1)); // spoustet sestupnou hranou
	
	/* nastaveni delice casovace
		- pro frekvenci 8MHz nesmi u zadne modulace presahnout doba klidu 8.192ms 
		  jinak jsou data povazovana za ztracena
		- pro frekvenci 16MHz nesmi u zadne modulace presahnout doba klidu 4.096ms 
		jinak jsou data povazovana za ztracena

		TODO: prozkoumat nastavebu prescaleru tak aby se mohlu zpracovavat i pomalejsi signaly
	*/		
	TCCR1B |= _BV(CS10); //  aktivovani citace F/1 (bez delicky)
	
	
	TIMSK |= _BV(TICIE1); // povoleni preruseni snimani hrany
	
	TCNT1 = 0;

	// !!!! Dioda musi bezet v impulznim zatizeni jinak dojde ke zniceni !!!
	IR_TX_OFF; // zhasnuti IR diody
}
