/*
	
 *************************************************************************************
 *                                                                                   *
 *              	MIRA_RC5 - Philips RC5 Protocol implementation                   *
 *                                                                                   *
 *************************************************************************************
 *                                                                                   *
 *  File:        	MIRA_RC5.c                                                       *
 *  Description:    C file of RC5 implmentation unit.                                *
 *                  This unit require MIRA unit.                                     *
 *                                                                                   *
 *  Author:   	 	Bc. Miroslav Skopal                                              *
 *  Date:     		23.5.2009                                                        *
 *  Platform: 		AVR (Tested on ATMega16 (fcpu = 16MHz)                           *
 *                                                                                   *
 *  License:		Licensed under GPLv2                                             *
 *                                                                                   *
 *  MIRA - Multiple InraRed Application Copyright (C) 2009 Miroslav Skopal           *
 *                                                                                   *
 *  This program is free software; you can redistribute it and/or                    *
 *  modify it under the terms of the GNU General Public License                      *
 *  as published by the Free Software Foundation; either version 2                   *
 *  of the License.                                                                  *
 *                                                                                   *
 *  This program is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 *  GNU General Public License for more details.                                     *
 *                                                                                   *
 *  You should have received a copy of the GNU General Public License                *
 *  along with this program; if not, write to the Free Software                      *
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 *                                                                                   * 
 *************************************************************************************

*/

#include <util/delay.h>
#include "MIRA.h"
#include "MIRA_RC5.h"

#define RC5_IDL		0
#define RC5_STB		1
#define RC5_ADR		2
#define RC5_CMD		3

char RC5_STATE = RC5_IDL;
char BurstCount = 0;
char RC5_ADDRESS;
char RC5_COMMAND;
char RC5_BITS;

char rxbit = 1;

void RC5_Recieve(unsigned short int us){

	// mezera mezi impulzi 36kHz (27.77us)		
	if ((us>20)&&(us<50)){	
		BurstCount++;

		// vyhodnoceni bitu
		if (BurstCount > 31){			
			BurstCount = 0;
			switch (RC5_STATE){
				case RC5_IDL:
					if (rxbit){
						RC5_STATE = RC5_STB; // prijat prvni start bit
						RC5_BITS = 1; 
						RC5_ADDRESS = 0;
						RC5_COMMAND = 0;
					}
				break;
				case RC5_STB:
					if (rxbit){
						RC5_STATE = RC5_ADR; // prijat druhy start bit
						RC5_BITS = 2;
					}else{
						RC5_STATE = RC5_IDL;
						RC5_BITS = 0;
					}
				break;
				case RC5_ADR: // prijimani adresy a togle bitu
					RC5_BITS++; // pocet prijatych bitu
					RC5_ADDRESS = (RC5_ADDRESS << 1) | rxbit;
					if (RC5_BITS == 8) RC5_STATE = RC5_CMD;
				break;
				case RC5_CMD: // prijimani commandu
					RC5_BITS++; // pocet prijatych bitu
					RC5_COMMAND = (RC5_COMMAND << 1) | rxbit;
					if (RC5_BITS == 14){
						// Prijat RC5 kod informuj o tom aplikaci					
						#ifdef RC5Recieve
							RC5Recieve;
						#endif
				
						RC5_BITS = 0;
						RC5_STATE = RC5_IDL;
						BurstCount = 0;
						rxbit = 0;
					}
				break;				
			}
			
			rxbit = !rxbit;
		}
		
		
	}else{
		if (BurstCount == 0){		
			if ((us > 850)&&(us < 970)){
				BurstCount = 1;				
				// prijmuti teto mezery znamena ze jsou prijmany 2 stejne bity po sobe
				if (RC5_BITS > 0) rxbit = !rxbit;
			}else
			if ((us > 1730)&&(us < 1930)){
				BurstCount = 1;			
				
				// tuto mezeru je mozne prijmou jenom v pripade ze predchozi bit byl 0 
				// nebo ze se system nachazi v IDLE  jinak je to chyba
				// tento stav nemuze nastat pred prijmem 4 bitu jinak je to chyba
				if (RC5_BITS < 3){
					BurstCount = 1;
					RC5_BITS = 0;
					RC5_STATE = RC5_IDL;
				}
				rxbit = 1;
			}else
			if ((us>850)&&(RC5_STATE == RC5_IDL)){
				BurstCount = 1;
				RC5_BITS = 0;
				rxbit = 1;					
			}else{
				// nepovoleny stav => vrat se do IDLE
				rxbit = 1;
				BurstCount = 1;
				RC5_BITS = 0;
				RC5_STATE = RC5_IDL;
			}
	
		}else{
			// nepovoleny stav => vrat se do IDLE
			BurstCount = 1                                                    ;
			RC5_BITS = 0;
			RC5_STATE = RC5_IDL;
			rxbit = 1;
		}
	}


			
}

// vysilani logicke 0
void RC5_SendBit(char bit){
	char i;
	bit &= 0x01;
	if (bit)_delay_ms(0.889);
	for (i=0;i<32;i++){
		IR_TX_ON;
		_delay_us(13.888);
		IR_TX_OFF;
		_delay_us(13.888);
	}	
	if (!bit)_delay_ms(0.889);
}

// Send RC5 command via Infrared diode
void RC5_Send(unsigned char Toggle, unsigned char Address, unsigned char Command){

	// vysilani Start bit = 2x 1
	RC5_SendBit(1);
	RC5_SendBit(1);

	// vysilani adresy
	RC5_SendBit(Toggle); // toogle bit je predavan jako 6. bit v adrese

	RC5_SendBit(Address >> 4);
	RC5_SendBit(Address >> 3);
	RC5_SendBit(Address >> 2);
	RC5_SendBit(Address >> 1);
	RC5_SendBit(Address	>> 0);

	// vysilani command
	RC5_SendBit(Command >> 5);
	RC5_SendBit(Command >> 4);
	RC5_SendBit(Command >> 3);
	RC5_SendBit(Command >> 2);
	RC5_SendBit(Command >> 1);
	RC5_SendBit(Command	>> 0);
}
